@@ -21,11 +21,11 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <glib-object.h>
+#include <gegl.h>
 
 #include "core/core-types.h"
 
-#include "base/base.h"
+#include "base/tile-swap.h"
 
 #include "core/gimp.h"
 #include "core/gimpparamspecs.h"
@@ -41,8 +41,7 @@
 #define BATCH_DEFAULT_EVAL_PROC   "plug-in-script-fu-eval"
 
 
-static gboolean  batch_exit_after_callback (Gimp          *gimp,
-                                            gboolean       kill_it);
+static gboolean  batch_exit_after_callback (Gimp          *gimp);
 static void      batch_run_cmd             (Gimp          *gimp,
                                             const gchar   *proc_name,
                                             GimpProcedure *procedure,
@@ -118,15 +117,23 @@ batch_run (Gimp         *gimp,
 }
 
 
+/*
+ * The purpose of this handler is to exit GIMP cleanly when the batch
+ * procedure calls the gimp-exit procedure. Without this callback, the
+ * message "batch command experienced an execution error" would appear
+ * and gimp would hang forever.
+ */
 static gboolean
-batch_exit_after_callback (Gimp     *gimp,
-                           gboolean  kill_it)
+batch_exit_after_callback (Gimp *gimp)
 {
   if (gimp->be_verbose)
-    g_print ("EXIT: %s\n", G_STRLOC);
+    g_print ("EXIT: %s\n", G_STRFUNC);
+
+  gegl_exit ();
+
+  /*  make sure that the swap files are removed before we quit */
+  tile_swap_exit ();
 
-  /*  make sure that the swap file is removed before we quit */
-  base_exit ();
   exit (EXIT_SUCCESS);
 
   return TRUE;
@@ -141,7 +148,8 @@ batch_run_cmd (Gimp          *gimp,
 {
   GValueArray *args;
   GValueArray *return_vals;
-  gint         i = 0;
+  GError      *error = NULL;
+  gint         i     = 0;
 
   args = gimp_procedure_get_arguments (procedure);
 
@@ -154,26 +162,45 @@ batch_run_cmd (Gimp          *gimp,
   return_vals =
     gimp_pdb_execute_procedure_by_name_args (gimp->pdb,
                                              gimp_get_user_context (gimp),
-                                             NULL,
+                                             NULL, &error,
                                              proc_name, args);
 
   switch (g_value_get_enum (&return_vals->values[0]))
     {
     case GIMP_PDB_EXECUTION_ERROR:
-      g_printerr ("batch command: experienced an execution error.\n");
+      if (error)
+        {
+          g_printerr ("batch command experienced an execution error: %s\n",
+                      error->message);
+        }
+      else
+        {
+          g_printerr ("batch command experienced an execution error\n");
+        }
       break;
 
     case GIMP_PDB_CALLING_ERROR:
-      g_printerr ("batch command: experienced a calling error.\n");
+      if (error)
+        {
+          g_printerr ("batch command experienced a calling error: %s\n",
+                      error->message);
+        }
+      else
+        {
+          g_printerr ("batch command experienced a calling error\n");
+        }
       break;
 
     case GIMP_PDB_SUCCESS:
-      g_printerr ("batch command: executed successfully.\n");
+      g_printerr ("batch command executed successfully\n");
       break;
     }
 
   g_value_array_free (return_vals);
   g_value_array_free (args);
 
+  if (error)
+    g_error_free (error);
+
   return;
 }
